# Find all variables in the current dataset that do not have any value labels.
# This illustrates the use of the spss.CreateXPathDictionary api, which produces the entire dictionary,
# and the Python xml processing facilities.
# An alternative way of processing the dictionary information is using XPath via the spss.EvaluateXPath api.

# This module is for use with Programming and Data Management for IBM SPSS Statistics 19.
# To make use of this module, copy it to your Python "site-packages" directory. This module makes use
# of the spss module, which is available once you've installed the Python Integration Plug-In.

#pubsmarker:import_start
from xml.sax.handler import ContentHandler
import xml.sax
import random, codecs, locale
import spss
#pubsmarker:import_end

# This class extracts the list of variables from the variable dictionary and determines which ones have 
# value labels by using the xml.sax module.
# The StartElement method of the content handler sees every element in the variable dictionary.
# It selects only variable name and value label information and updates the sets as they appear.
# These are later retrieved from the handler object.

# It may be helpful in working through this example to look at actual dictionary xml. You can
# produce a file of this using something like
#   spss.GetXmlUtf16(tag, filespec)
# where tag identifies the output of spss.CreateXPathDictionary in the XML workspace.

__author__ =  'SPSS Inc.'
__version__=  '2.0.0'

#pubsmarker:labelhandler_start
class valueLabelHandler(ContentHandler):
    """Create two sets: one listing all variable names and
    the other listing variables with value labels"""
    def __init__(self):
        self.varset = set()
        self.vallabelset = set()
    def startElement(self, name, attr):
        if name == u"variable":
            self.varset.add(attr.getValue(u"name"))
        elif name == u"valueLabelVariable":
            self.vallabelset.add(attr.getValue(u"name"))
#pubsmarker:labelhandler_end

# Use the class above to find variables without value labels

#pubsmarker:varsnolabels_start
def FindVarsWithoutValueLabels():
    handler = valueLabelHandler()
    tag = "D"+ str(random.uniform(0,1))
    spss.CreateXPathDictionary(tag)

    # Retrieve and parse the variable dictionary
    xml.sax.parseString(spss.GetXmlUtf16(tag),handler)
    spss.DeleteXPathHandle(tag)

    # Print a list of variables in varset that aren't in vallabelset
    nolabelset = handler.varset.difference(handler.vallabelset)
    if nolabelset:
        print "The following variables have no value labels:"
        print "\n".join([v for v in nolabelset])
    else:
        print "All variables in this dataset have at least one value label."
#pubsmarker:varsnolabels_end    
